@@ -31,25 +31,39 @@
 #include <cstdio>
 #include <cstdlib>
 #include <cmath>
-#include <cstdarg>
 #include <iostream>
 #include <vector>
+#include <memory>
 
-#include "dassert.h"
-#include "typedesc.h"
-#include "filesystem.h"
-#include "plugin.h"
-#include "thread.h"
-#include "strutil.h"
-
-#include "imageio.h"
+#include <OpenImageIO/dassert.h>
+#include <OpenImageIO/typedesc.h>
+#include <OpenImageIO/filesystem.h>
+#include <OpenImageIO/plugin.h>
+#include <OpenImageIO/thread.h>
+#include <OpenImageIO/strutil.h>
+#include <OpenImageIO/imageio.h>
+#include <OpenImageIO/deepdata.h>
 #include "imageio_pvt.h"
 
 
-OIIO_NAMESPACE_ENTER
-{
+OIIO_NAMESPACE_BEGIN
     using namespace pvt;
 
+
+
+ImageOutput::ImageOutput ()
+    : m_threads(0)
+{
+}
+
+
+
+ImageOutput::~ImageOutput ()
+{
+}
+
+
+
 bool
 ImageOutput::write_scanline (int y, int z, TypeDesc format,
                              const void *data, stride_t xstride)
@@ -111,7 +125,7 @@ bool ImageOutput::write_tiles (int xbegin, int xend, int ybegin, int yend,
 
     bool ok = true;
     stride_t pixelsize = format.size() * m_spec.nchannels;
-    std::vector<char> buf;
+    std::unique_ptr<char[]> buf;
     for (int z = zbegin;  z < zend;  z += std::max(1,m_spec.tile_depth)) {
         int zd = std::min (zend-z, m_spec.tile_depth);
         for (int y = ybegin;  y < yend;  y += m_spec.tile_height) {
@@ -128,8 +142,9 @@ bool ImageOutput::write_tiles (int xbegin, int xend, int ybegin, int yend,
                     ok &= write_tile (x, y, z, format, tilestart,
                                      xstride, ystride, zstride);
                 } else {
-                    buf.resize (pixelsize * m_spec.tile_pixels());
-                    OIIO_NAMESPACE::copy_image (m_spec.nchannels, xw, yh, zd,
+                    if (! buf.get())
+                        buf.reset (new char [pixelsize * m_spec.tile_pixels()]);
+                    OIIO::copy_image (m_spec.nchannels, xw, yh, zd,
                                 tilestart, pixelsize, xstride, ystride, zstride,
                                 &buf[0], pixelsize, pixelsize*m_spec.tile_width,
                                 pixelsize*m_spec.tile_pixels());
@@ -157,6 +172,50 @@ ImageOutput::write_rectangle (int xbegin, int xend, int ybegin, int yend,
 
 
 
+bool
+ImageOutput::write_deep_scanlines (int ybegin, int yend, int z,
+                                   const DeepData &deepdata)
+{
+    return false;  // default: doesn't support deep images
+}
+
+
+
+bool
+ImageOutput::write_deep_tiles (int xbegin, int xend, int ybegin, int yend,
+                               int zbegin, int zend,
+                               const DeepData &deepdata)
+{
+    return false;  // default: doesn't support deep images
+}
+
+
+
+bool
+ImageOutput::write_deep_image (const DeepData &deepdata)
+{
+    if (m_spec.depth > 1) {
+        error ("write_deep_image is not supported for volume (3D) images.");
+        return false;
+        // FIXME? - not implementing 3D deep images for now.  The only
+        // format that supports deep images at this time is OpenEXR, and
+        // it doesn't support volumes.
+    }
+    if (m_spec.tile_width) {
+        // Tiled image
+        return write_deep_tiles (m_spec.x, m_spec.x+m_spec.width,
+                                 m_spec.y, m_spec.y+m_spec.height,
+                                 m_spec.z, m_spec.z+m_spec.depth,
+                                 deepdata);
+    } else {
+        // Scanline image
+        return write_deep_scanlines (m_spec.y, m_spec.y+m_spec.height, 0,
+                                     deepdata);
+    }
+}
+
+
+
 int
 ImageOutput::send_to_output (const char *format, ...)
 {
@@ -176,16 +235,13 @@ ImageOutput::send_to_client (const char *format, ...)
 
 
 void
-ImageOutput::error (const char *format, ...)
+ImageOutput::append_error (const std::string& message) const
 {
-    va_list ap;
-    va_start (ap, format);
     ASSERT (m_errmessage.size() < 1024*1024*16 &&
             "Accumulated error messages > 16MB. Try checking return codes!");
     if (m_errmessage.size())
         m_errmessage += '\n';
-    m_errmessage += Strutil::vformat (format, ap);
-    va_end (ap);
+    m_errmessage += message;
 }
 
 
@@ -193,10 +249,13 @@ ImageOutput::error (const char *format, ...)
 const void *
 ImageOutput::to_native_scanline (TypeDesc format,
                                  const void *data, stride_t xstride,
-                                 std::vector<unsigned char> &scratch)
+                                 std::vector<unsigned char> &scratch,
+                                 unsigned int dither,
+                                 int yorigin, int zorigin)
 {
     return to_native_rectangle (0, m_spec.width, 0, 1, 0, 1, format, data,
-                                xstride, 0, 0, scratch);
+                                xstride, 0, 0, scratch, dither,
+                                m_spec.x, yorigin, zorigin);
 }
 
 
@@ -204,11 +263,14 @@ ImageOutput::to_native_scanline (TypeDesc format,
 const void *
 ImageOutput::to_native_tile (TypeDesc format, const void *data,
                              stride_t xstride, stride_t ystride, stride_t zstride,
-                             std::vector<unsigned char> &scratch)
+                             std::vector<unsigned char> &scratch,
+                             unsigned int dither,
+                             int xorigin, int yorigin, int zorigin)
 {
     return to_native_rectangle (0, m_spec.tile_width, 0, m_spec.tile_height,
                                 0, std::max(1,m_spec.tile_depth), format, data,
-                                xstride, ystride, zstride, scratch);
+                                xstride, ystride, zstride, scratch,
+                                dither, xorigin, yorigin, zorigin);
 }
 
 
@@ -218,7 +280,9 @@ ImageOutput::to_native_rectangle (int xbegin, int xend, int ybegin, int yend,
                                   int zbegin, int zend,
                                   TypeDesc format, const void *data,
                                   stride_t xstride, stride_t ystride, stride_t zstride,
-                                  std::vector<unsigned char> &scratch)
+                                  std::vector<unsigned char> &scratch,
+                                  unsigned int dither,
+                                  int xorigin, int yorigin, int zorigin)
 {
     // native_pixel_bytes is the size of a pixel in the FILE, including
     // the per-channel format, if specified when the file was opened.
@@ -232,6 +296,12 @@ ImageOutput::to_native_rectangle (int xbegin, int xend, int ybegin, int yend,
     // native_data is true if the user is passing data in the native format
     bool native_data = (format == TypeDesc::UNKNOWN ||
                         (format == m_spec.format && !perchanfile));
+    stride_t input_pixel_bytes = native_data ? native_pixel_bytes
+                                             : stride_t(format.size()*m_spec.nchannels);
+    // If user is passing native data and it's all one type, go ahead and
+    // set format correctly.
+    if (format == TypeDesc::UNKNOWN && !perchanfile)
+        format = m_spec.format;
     // If the user is passing native data and they've left xstride set
     // to Auto, then we know it's the native pixel size.
     if (native_data && xstride == AutoStride)
@@ -267,7 +337,7 @@ ImageOutput::to_native_rectangle (int xbegin, int xend, int ybegin, int yend,
 
     imagesize_t rectangle_pixels = width * height * depth;
     imagesize_t rectangle_values = rectangle_pixels * m_spec.nchannels;
-    imagesize_t rectangle_bytes = rectangle_pixels * native_pixel_bytes;
+    imagesize_t native_rectangle_bytes = rectangle_pixels * native_pixel_bytes;
 
     // Cases to handle:
     // 1. File has per-channel data, user passes native data -- this has
@@ -285,7 +355,7 @@ ImageOutput::to_native_rectangle (int xbegin, int xend, int ybegin, int yend,
         }
         ASSERT (format != TypeDesc::UNKNOWN);
         ASSERT (m_spec.channelformats.size() == (size_t)m_spec.nchannels);
-        scratch.resize (rectangle_bytes);
+        scratch.resize (native_rectangle_bytes);
         size_t offset = 0;
         for (int c = 0;  c < m_spec.nchannels;  ++c) {
             TypeDesc chanformat = m_spec.channelformats[c];
@@ -304,11 +374,13 @@ ImageOutput::to_native_rectangle (int xbegin, int xend, int ybegin, int yend,
     // The remaining code is where all channels in the file have the
     // same data type, which may or may not be what the user passed in
     // (cases #3 and #4 above).
-    imagesize_t contiguoussize = contiguous ? 0 : rectangle_values * native_pixel_bytes;
+    imagesize_t contiguoussize = contiguous ? 0 : rectangle_values * input_pixel_bytes;
     contiguoussize = (contiguoussize+3) & (~3); // Round up to 4-byte boundary
     DASSERT ((contiguoussize & 3) == 0);
     imagesize_t floatsize = rectangle_values * sizeof(float);
-    scratch.resize (contiguoussize + floatsize + rectangle_bytes);
+    bool do_dither = (dither && format.is_floating_point() &&
+                      m_spec.format.basetype == TypeDesc::UINT8);
+    scratch.resize (contiguoussize + floatsize + native_rectangle_bytes);
 
     // Force contiguity if not already present
     if (! contiguous) {
@@ -321,19 +393,32 @@ ImageOutput::to_native_rectangle (int xbegin, int xend, int ybegin, int yend,
     // will always preserve enough precision.
     const float *buf;
     if (format == TypeDesc::FLOAT) {
-        // Already in float format -- leave it as-is.
-        buf = (float *)data;
+        if (! do_dither) {
+            // Already in float format and no dither -- leave it as-is.
+            buf = (float *)data;
+        } else {
+            // Need to make a copy, even though it's already float, so the
+            // dither doesn't overwrite the caller's data.
+            buf = (float *)&scratch[contiguoussize];
+            memcpy ((float *)buf, data, floatsize);
+        }
     } else {
-        // Convert to from 'format' to float.
+        // Convert from 'format' to float.
         buf = convert_to_float (data, (float *)&scratch[contiguoussize],
-                                rectangle_values, format);
+                                (int)rectangle_values, format);
     }
-    
+
+    if (do_dither) {
+        stride_t pixelsize = m_spec.nchannels * sizeof(float);
+        OIIO::add_dither (m_spec.nchannels, width, height, depth, (float *)buf,
+                          pixelsize, pixelsize*width, pixelsize*width*height,
+                          1.0f/255.0f, m_spec.alpha_channel, m_spec.z_channel,
+                          dither, 0, xorigin, yorigin, zorigin);
+    }
+
     // Convert from float to native format.
-    return convert_from_float (buf, &scratch[contiguoussize+floatsize], 
-                       rectangle_values, m_spec.quant_black, m_spec.quant_white,
-                       m_spec.quant_min, m_spec.quant_max,
-                       m_spec.format);
+    return parallel_convert_from_float (buf, &scratch[contiguoussize+floatsize], 
+                                        rectangle_values, m_spec.format);
 }
 
 
@@ -424,23 +509,112 @@ ImageOutput::copy_image (ImageInput *in)
     // but there are some exceptions (like in FITS plugin)
     // when we want to do this. Because 0x0 means there is no image
     // data in the file, we simply return true so the application thought
-    // that everything went right
+    // that everything went right.
     if (! spec().image_bytes())
         return true;
 
+    if (spec().deep) {
+        // Special case for ''deep'' images
+        DeepData deepdata;
+        bool ok = in->read_native_deep_image (deepdata);
+        if (ok)
+            ok = write_deep_image (deepdata);
+        else
+            error ("%s", in->geterror());  // copy err from in to out
+        return ok;
+    }
+
     // Naive implementation -- read the whole image and write it back out.
     // FIXME -- a smarter implementation would read scanlines or tiles at
     // a time, to minimize mem footprint.
     bool native = supports("channelformats") && inspec.channelformats.size();
     TypeDesc format = native ? TypeDesc::UNKNOWN : inspec.format;
-    std::vector<char> pixels (inspec.image_bytes(native));
+    std::unique_ptr<char[]> pixels (new char [inspec.image_bytes(native)]);
     bool ok = in->read_image (format, &pixels[0]);
-    if (!ok)
-        error ("%s", in->geterror().c_str());  // copy err from in to out
     if (ok)
         ok = write_image (format, &pixels[0]);
+    else
+        error ("%s", in->geterror());  // copy err from in to out
     return ok;
 }
 
+
+
+bool
+ImageOutput::copy_to_image_buffer (int xbegin, int xend, int ybegin, int yend,
+                                   int zbegin, int zend, TypeDesc format,
+                                   const void *data, stride_t xstride,
+                                   stride_t ystride, stride_t zstride,
+                                   void *image_buffer, TypeDesc buf_format)
+{
+    const ImageSpec &spec (this->spec());
+    if (buf_format == TypeDesc::UNKNOWN)
+        buf_format = spec.format;
+    spec.auto_stride (xstride, ystride, zstride, format, spec.nchannels,
+                      spec.width, spec.height);
+    stride_t buf_xstride = spec.nchannels * buf_format.size();
+    stride_t buf_ystride = buf_xstride * spec.width;
+    stride_t buf_zstride = buf_ystride * spec.height;
+    stride_t offset = (xbegin-spec.x)*buf_xstride
+                    + (ybegin-spec.y)*buf_ystride
+                    + (zbegin-spec.z)*buf_zstride;
+    int width = xend-xbegin, height = yend-ybegin, depth = zend-zbegin;
+    imagesize_t npixels = imagesize_t(width) * imagesize_t(height) * imagesize_t(depth);
+
+    // Add dither if requested -- requires making a temporary staging area
+    std::unique_ptr<float[]> ditherarea;
+    unsigned int dither = spec.get_int_attribute ("oiio:dither", 0);
+    if (dither && format.is_floating_point() &&
+            buf_format.basetype == TypeDesc::UINT8) {
+        stride_t pixelsize = spec.nchannels * sizeof(float);
+        ditherarea.reset (new float [pixelsize * npixels]);
+        OIIO::convert_image (spec.nchannels, width, height, depth,
+                             data, format, xstride, ystride, zstride,
+                             ditherarea.get(), TypeDesc::FLOAT,
+                             pixelsize, pixelsize*width,
+                             pixelsize*width*height);
+        data = ditherarea.get();
+        format = TypeDesc::FLOAT;
+        xstride = pixelsize;
+        ystride = xstride * width;
+        zstride = ystride * height;
+        float ditheramp = spec.get_float_attribute ("oiio:ditheramplitude", 1.0f/255.0f);
+        OIIO::add_dither (spec.nchannels, width, height, depth, (float *)data,
+                          pixelsize, pixelsize*width, pixelsize*width*height,
+                          ditheramp, spec.alpha_channel, spec.z_channel,
+                          dither, 0, xbegin, ybegin, zbegin);
+    }
+
+    return OIIO::convert_image (spec.nchannels, width, height, depth,
+                                data, format, xstride, ystride, zstride,
+                                (char *)image_buffer + offset, buf_format,
+                                buf_xstride, buf_ystride, buf_zstride);
 }
-OIIO_NAMESPACE_EXIT
+
+
+
+bool
+ImageOutput::copy_tile_to_image_buffer (int x, int y, int z, TypeDesc format,
+                                        const void *data, stride_t xstride,
+                                        stride_t ystride, stride_t zstride,
+                                        void *image_buffer,
+                                        TypeDesc buf_format)
+{
+    if (! m_spec.tile_width || ! m_spec.tile_height) {
+        error ("Called write_tile for non-tiled image.");
+        return false;
+    }
+    const ImageSpec &spec (this->spec());
+    spec.auto_stride (xstride, ystride, zstride, format, spec.nchannels,
+                      spec.tile_width, spec.tile_height);
+    int xend = std::min (x+spec.tile_width,  spec.x+spec.width);
+    int yend = std::min (y+spec.tile_height, spec.y+spec.height);
+    int zend = std::min (z+spec.tile_depth,  spec.z+spec.depth);
+    return copy_to_image_buffer (x, xend, y, yend, z, zend,
+                                 format, data, xstride, ystride, zstride,
+                                 image_buffer, buf_format);
+}
+
+
+
+OIIO_NAMESPACE_END
